/**
 * @module botbuilder-adapter-vonage
 *
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 *
 * Connect [Botkit](https://www.npmjs.com/package/botkit) or [BotBuilder](https://www.npmjs.com/package/botbuilder) to Vonage's SMS service.
 */
import { Activity, BotAdapter, TurnContext, ConversationReference, ResourceResponse } from "botbuilder";
import { VonageBotWorker } from "./botworker";
import { CredentialsObject } from "@vonage/server-sdk";
/**
 * Pass to the VonageAdapter the the Vonage SDK constructor the required parameter
 * credentials (apiKey, apiSecret, applicationId, privateKey)
 */
export interface VonageAdapterOptions extends CredentialsObject {
    /**
     * The phone number initiating the conversation, in the format 1XXXYYYZZZZ
     */
    to_number: string;
    /**
     * The Bot, the phone number associated with this Vonage app (virtual number), in the format 1XXXYYYZZZZ
     */
    from_number: string;
    /**
     * Allow the adapter to startup without a complete configuration.
     * This is risky as it may result in a non-functioning or insecure adapter.
     * This should only be used when getting started.
     */
    enable_incomplete?: boolean;
}
export declare class VonageAdapter extends BotAdapter {
    /**
     * Name used by Botkit plugin loader
     * @ignore
     */
    name: string;
    /**
     * Object containing one or more Botkit middlewares to bind automatically.
     * @ignore
     */
    middlewares: any;
    /**
     * A specialized BotWorker for Botkit that exposes Vonage specific extension methods.
     * @ignore
     */
    botkit_worker: typeof VonageBotWorker;
    private options;
    private api;
    /**
     * Create an adapter to handle incoming messages from Vonage's SMS service and translate them into a standard format for processing by your bot.
     * GO HERE [Vonage SDK](https://github.com/Vonage/vonage-node-sdk)
     * Use with Botkit:
     *```js
     * const adapter = new VonageAdapter({
     *    to_number: process.env.TO_NUMBER,       // The phone number initiating the conversation, in the format 1XXXYYYZZZZ
     *    from_number: process.env.FROM_NUMBER    // The Bot, the phone number associated with this Vonage app (virtual number), in the format 1XXXYYYZZZZ
     *    apiKey: API_KEY,                        // If applicationId and privateKey are present, apiKey is optional.
     *    apiSecret: API_SECRET,                  // If applicationId and privateKey are present, apiSecret is optional.
     *    applicationId: APP_ID,              // (optional) The Vonage API Application ID to be used when creating JWTs.
     *    privateKey: PRIVATE_KEY_PATH,       // (optional) The Private Key to be used when creating JWTs.
     *    signatureSecret: SIGNATURE_SECRET,  // (optional) API signature secret from Vonage API, used for signing SMS message requests
     *    signatureMethod: SIGNATURE_METHOD   // (optional) signature method matching the one you gave Vonage API, used for signing SMS message requests. Must be one of "md5hash", "md5", "sha1", "sha256", or "sha512"
     *  }, options );                         // (optional) Additional options for the constructor.
     *
     * const controller = new Botkit({
     *    adapter: adapter,
     *    // ... other configuration options
     * });
     * ```
     *
     * Use with BotBuilder:
     *```js
     * const adapter = new VonageAdapter({
     *    to_number: process.env.TO_NUMBER,       // The phone number initiating the conversation, in the format 1XXXYYYZZZZ
     *    from_number: process.env.FROM_NUMBER    // The Bot, the phone number associated with this Vonage app (virtual number), in the format 1XXXYYYZZZZ
     *    apiKey: API_KEY,                        // If applicationId and privateKey are present, apiKey is optional.
     *    apiSecret: API_SECRET,                  // If applicationId and privateKey are present, apiSecret is optional.
     *    applicationId: APP_ID,              // (optional) The Vonage API Application ID to be used when creating JWTs.
     *    privateKey: PRIVATE_KEY_PATH,       // (optional) The Private Key to be used when creating JWTs.
     *    signatureSecret: SIGNATURE_SECRET,  // (optional) API signature secret from Vonage API, used for signing SMS message requests
     *    signatureMethod: SIGNATURE_METHOD   // (optional) signature method matching the one you gave Vonage API, used for signing SMS message requests. Must be one of "md5hash", "md5", "sha1", "sha256", or "sha512"
     *  }, options );                         // (optional) Additional options for the constructor.
     *
     * // set up restify...
     * const server = restify.createServer();
     * server.use(restify.plugins.bodyParser());
     * server.post('/api/messages', (req, res) => {
     *      adapter.processActivity(req, res, async(context) => {
     *          // do your bot logic here!
     *      });
     * });
     * ```
     *
     * @param options An object containing API credentials, a webhook verification token and other options
     */
    constructor(options: VonageAdapterOptions);
    /**
     * Converts a BotBuilder Activity Object into a outbound ready Vonage Message ready for the API.
     * @param activity A BotBuilder Activity object
     * @returns a Vonage message object with {body, from, to}
     */
    private activityToVonage;
    /** Message API <<< SendActiviy <<< Bot
     * Standard BotBuilder adapter method to send a message from the bot to the messaging API.
     * [BotBuilder reference docs](https://docs.microsoft.com/en-us/javascript/api/botbuilder-core/botadapter?view=botbuilder-ts-latest#sendactivities).
     * @param context A TurnContext representing the current incoming message and environment. (Not used)
     * @param activities An array of outgoing activities to be sent back to the messaging API.
     */
    sendActivities(context: TurnContext, activities: Partial<Activity>[]): Promise<ResourceResponse[]>;
    /**
     * Vonage SMS adapter does not support updateActivity.
     * @ignore
     */
    updateActivity(context: TurnContext, activity: Partial<Activity>): Promise<void>;
    /**
     * Vonage SMS adapter does not support deleteActivity.
     * @ignore
     */
    deleteActivity(context: TurnContext, reference: Partial<ConversationReference>): Promise<void>;
    /**
     * Standard BotBuilder adapter method for continuing an existing conversation based on a conversation reference.
     * [BotBuilder reference docs](https://docs.microsoft.com/en-us/javascript/api/botbuilder-core/botadapter?view=botbuilder-ts-latest#continueconversation)
     * @param reference A conversation reference to be applied to future messages.
     * @param logic A bot logic function that will perform continuing action in the form `async(context) => { ... }`
     */
    continueConversation(reference: Partial<ConversationReference>, logic: (context: TurnContext) => Promise<void>): Promise<void>;
    /** Message API >>> ProcessActivity >>> Bot
     * Accept an incoming webhook request and convert it into a TurnContext which can be
     * processed by the bot's logic.
     * @param req A request object from Restify or Express
     * @param res A response object from Restify or Express
     * @param logic A bot logic function in the form `async(context) => { ... }`
     */
    processActivity(req: any, res: any, logic: (context: TurnContext) => Promise<void>): Promise<void>;
    /** SMS >>> ProcessActivy >>>
     * Verify the signature of an incoming webhook request as originating from Vonage.
     * @returns If signature is valid, returns true. Otherwise, sends a 400 error status via http response and then returns false.
     */
    private verifySignature;
}
