/**
 * @module botbuilder-adapter-vonage
 *
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 *
 * Connect [Botkit](https://www.npmjs.com/package/botkit) or [BotBuilder](https://www.npmjs.com/package/botbuilder) to Vonage's SMS service.
 */
import {
  Activity,
  ActivityTypes,
  BotAdapter,
  TurnContext,
  ConversationReference,
  ResourceResponse,
} from "botbuilder";
import { VonageBotWorker } from "./botworker";
import Vonage, { CredentialsObject } from "@vonage/server-sdk";
import * as Debug from "debug";
const debug = Debug("botkit:vonage");
import { promisify } from "util";
/**
 * Pass to the VonageAdapter the the Vonage SDK constructor the required parameter
 * credentials (apiKey, apiSecret, applicationId, privateKey)
 */
export interface VonageAdapterOptions extends CredentialsObject {
  /**
   * The phone number initiating the conversation, in the format 1XXXYYYZZZZ
   */
  to_number: string;
  /**
   * The Bot, the phone number associated with this Vonage app (virtual number), in the format 1XXXYYYZZZZ
   */
  from_number: string;
  /**
   * Allow the adapter to startup without a complete configuration.
   * This is risky as it may result in a non-functioning or insecure adapter.
   * This should only be used when getting started.
   */
  enable_incomplete?: boolean;
}
export class VonageAdapter extends BotAdapter {
  /**
   * Name used by Botkit plugin loader
   * @ignore
   */
  public name = "Vonage Adapter";

  /**
   * Object containing one or more Botkit middlewares to bind automatically.
   * @ignore
   */
  public middlewares;

  /**
   * A specialized BotWorker for Botkit that exposes Vonage specific extension methods.
   * @ignore
   */
  public botkit_worker = VonageBotWorker;

  private options: VonageAdapterOptions;
  private api: Vonage; // Vonage api

  /**
   * Create an adapter to handle incoming messages from Vonage's SMS service and translate them into a standard format for processing by your bot.
   * GO HERE [Vonage SDK](https://github.com/Vonage/vonage-node-sdk)
   * Use with Botkit:
   *```js
   * const adapter = new VonageAdapter({
   *    to_number: process.env.TO_NUMBER,       // The phone number initiating the conversation, in the format 1XXXYYYZZZZ
   *    from_number: process.env.FROM_NUMBER    // The Bot, the phone number associated with this Vonage app (virtual number), in the format 1XXXYYYZZZZ
   *    apiKey: API_KEY,                        // If applicationId and privateKey are present, apiKey is optional.
   *    apiSecret: API_SECRET,                  // If applicationId and privateKey are present, apiSecret is optional.
   *    applicationId: APP_ID,              // (optional) The Vonage API Application ID to be used when creating JWTs.
   *    privateKey: PRIVATE_KEY_PATH,       // (optional) The Private Key to be used when creating JWTs.
   *    signatureSecret: SIGNATURE_SECRET,  // (optional) API signature secret from Vonage API, used for signing SMS message requests
   *    signatureMethod: SIGNATURE_METHOD   // (optional) signature method matching the one you gave Vonage API, used for signing SMS message requests. Must be one of "md5hash", "md5", "sha1", "sha256", or "sha512"
   *  }, options );                         // (optional) Additional options for the constructor.
   *
   * const controller = new Botkit({
   *    adapter: adapter,
   *    // ... other configuration options
   * });
   * ```
   *
   * Use with BotBuilder:
   *```js
   * const adapter = new VonageAdapter({
   *    to_number: process.env.TO_NUMBER,       // The phone number initiating the conversation, in the format 1XXXYYYZZZZ
   *    from_number: process.env.FROM_NUMBER    // The Bot, the phone number associated with this Vonage app (virtual number), in the format 1XXXYYYZZZZ
   *    apiKey: API_KEY,                        // If applicationId and privateKey are present, apiKey is optional.
   *    apiSecret: API_SECRET,                  // If applicationId and privateKey are present, apiSecret is optional.
   *    applicationId: APP_ID,              // (optional) The Vonage API Application ID to be used when creating JWTs.
   *    privateKey: PRIVATE_KEY_PATH,       // (optional) The Private Key to be used when creating JWTs.
   *    signatureSecret: SIGNATURE_SECRET,  // (optional) API signature secret from Vonage API, used for signing SMS message requests
   *    signatureMethod: SIGNATURE_METHOD   // (optional) signature method matching the one you gave Vonage API, used for signing SMS message requests. Must be one of "md5hash", "md5", "sha1", "sha256", or "sha512"
   *  }, options );                         // (optional) Additional options for the constructor.
   *
   * // set up restify...
   * const server = restify.createServer();
   * server.use(restify.plugins.bodyParser());
   * server.post('/api/messages', (req, res) => {
   *      adapter.processActivity(req, res, async(context) => {
   *          // do your bot logic here!
   *      });
   * });
   * ```
   *
   * @param options An object containing API credentials, a webhook verification token and other options
   */
  public constructor(options: VonageAdapterOptions) {
    super();

    this.options = options;

    if (!options.apiKey || !options.applicationId) {
      const err =
        "Either apiKey or applicationId is required part of the configuration";
      if (!this.options.enable_incomplete) {
        throw new Error(err);
      } else {
        console.error(err);
      }
    }

    if (!options.apiSecret || !options.privateKey) {
      const err =
        "Either apiKey or applicationId is required part of the configuration";
      if (!this.options.enable_incomplete) {
        throw new Error(err);
      } else {
        console.error(err);
      }
    }

    if (!options.to_number || !options.from_number) {
      const err =
        "Both to_number and from_number are required parts of the configuration";
      if (!this.options.enable_incomplete) {
        throw new Error(err);
      } else {
        console.error(err);
      }
    }

    if (this.options.enable_incomplete) {
      const warning = [
        "",
        "****************************************************************************************",
        "* WARNING: Your adapter may be running with an incomplete/unsafe configuration.        *",
        "* - Ensure all required configuration options are present                              *",
        '* - Disable the "enable_incomplete" option!                                            *',
        "****************************************************************************************",
        "",
      ];
      console.warn(warning.join("\n"));
    }

    try {
      this.api = new Vonage(this.options);
    } catch (err) {
      if (err) {
        if (!this.options.enable_incomplete) {
          throw new Error(err);
        } else {
          console.error(err);
        }
      }
    }

    this.middlewares = {
      spawn: [
        async (bot, next): Promise<void> => {
          // make the Vonage API available to the bot instance.
          bot.api = this.api;
          next();
        },
      ],
    };
  }

  /**
   * Converts a BotBuilder Activity Object into a outbound ready Vonage Message ready for the API.
   * @param activity A BotBuilder Activity object
   * @returns a Vonage message object with {body, from, to}
   */
  private activityToVonage(activity: Partial<Activity>): any {
    const message = {
      message: activity.channelData.message.text,
      from: activity.channelData.from.number, // this.options.vonage_number
      to: activity.channelData.to.number, // activity.conversation.id
    };

    // if (activity.channelData && activity.channelData.message.text) {
    //   message.message = activity.channelData.message.text;
    // }
    return message;
  }

  /** Message API <<< SendActiviy <<< Bot
   * Standard BotBuilder adapter method to send a message from the bot to the messaging API.
   * [BotBuilder reference docs](https://docs.microsoft.com/en-us/javascript/api/botbuilder-core/botadapter?view=botbuilder-ts-latest#sendactivities).
   * @param context A TurnContext representing the current incoming message and environment. (Not used)
   * @param activities An array of outgoing activities to be sent back to the messaging API.
   */
  public async sendActivities(
    context: TurnContext,
    activities: Partial<Activity>[]
  ): Promise<ResourceResponse[]> {
    const sendMessageOverChannel = promisify(this.api.channel.send);

    const responses = [];
    for (let a = 0; a < activities.length; a++) {
      const activity = activities[a];
      if (
        activity.type === ActivityTypes.Message ||
        activity.type === ActivityTypes.Event
      ) {
        const message = this.activityToVonage(activity as Activity);
        try {
          const { message_uuid } = await sendMessageOverChannel(
            activity.channelData.to.number,
            activity.channelData.from.number,
            message
          );
          responses.push({ id: message_uuid });
        } catch (err) {
          debug("Error sending message over channel", err);
        }
      } else {
        debug(
          "Unknown message type encountered in sendActivities: ",
          activity.type
        );
      }
    }

    return responses;
  }

  /**
   * Vonage SMS adapter does not support updateActivity.
   * @ignore
   */
  // eslint-disable-next-line
  public async updateActivity(
    context: TurnContext,
    activity: Partial<Activity>
  ): Promise<void> {
    debug("Vonage SMS does not support updating activities.");
  }

  /**
   * Vonage SMS adapter does not support deleteActivity.
   * @ignore
   */
  // eslint-disable-next-line
  public async deleteActivity(
    context: TurnContext,
    reference: Partial<ConversationReference>
  ): Promise<void> {
    debug("Vonage SMS does not support deleting activities.");
  }

  /**
   * Standard BotBuilder adapter method for continuing an existing conversation based on a conversation reference.
   * [BotBuilder reference docs](https://docs.microsoft.com/en-us/javascript/api/botbuilder-core/botadapter?view=botbuilder-ts-latest#continueconversation)
   * @param reference A conversation reference to be applied to future messages.
   * @param logic A bot logic function that will perform continuing action in the form `async(context) => { ... }`
   */
  public async continueConversation(
    reference: Partial<ConversationReference>,
    logic: (context: TurnContext) => Promise<void>
  ): Promise<void> {
    const request = TurnContext.applyConversationReference(
      { type: "event", name: "continueConversation" },
      reference,
      true
    );
    const context = new TurnContext(this, request);

    return this.runMiddleware(context, logic);
  }
  /** Message API >>> ProcessActivity >>> Bot
   * Accept an incoming webhook request and convert it into a TurnContext which can be
   * processed by the bot's logic.
   * @param req A request object from Restify or Express
   * @param res A response object from Restify or Express
   * @param logic A bot logic function in the form `async(context) => { ... }`
   */
  public async processActivity(
    req,
    res,
    logic: (context: TurnContext) => Promise<void>
  ): Promise<void> {
    if ((await this.verifySignature(req, res)) === true) {
      const event = req.body;

      const activity = {
        id: event.message_uuid,
        timestamp: new Date(),
        channelId: "vonage-sms",
        conversation: {
          id: event.from.number,
        },
        from: {
          id: event.from.number,
        },
        recipient: {
          id: event.to.number,
        },
        text: event.message.content.text,
        channelData: event,
        type: ActivityTypes.Message, // ?????
      };

      // Detect attachments
      if (event.NumMedia && parseInt(event.NumMedia) > 0) {
        // specify a different event type for Botkit
        activity.channelData.botkitEventType = "picture_message";
      }

      // create a conversation reference
      const context = new TurnContext(this, activity as Activity);

      context.turnState.set("httpStatus", 200);

      await this.runMiddleware(context, logic);

      // send http response back
      res.status(context.turnState.get("httpStatus"));
      if (context.turnState.get("httpBody")) {
        res.send(context.turnState.get("httpBody"));
      } else {
        res.end();
      }
    }
  }

  /** SMS >>> ProcessActivy >>>
   * Verify the signature of an incoming webhook request as originating from Vonage.
   * @returns If signature is valid, returns true. Otherwise, sends a 400 error status via http response and then returns false.
   */
  private async verifySignature(req, res): Promise<any> {
    let vonageSignature;
    let validation_url;

    // Restify style
    if (!req.headers) {
      vonageSignature = req.header("X-OPENTOK-AUTH");

      validation_url =
        (req.headers["x-forwarded-proto"] || req.isSecure()
          ? "https"
          : "http") +
        "://" +
        req.headers.host +
        req.url;
    } else {
      // express style
      vonageSignature = req.headers["x-vonage-signature"];

      validation_url =
        (req.headers["x-forwarded-proto"] || req.protocol) +
        "://" +
        req.hostname +
        req.originalUrl;
    }

    if (
      vonageSignature
      // && Vonage.validateRequest(
      //   // Vonage.validateRequest
      //   this.options.auth_token,
      //   vonageSignature,
      //   validation_url,
      //   req.body
      // )
    ) {
      return true;
    } else {
      debug("Signature verification failed, Ignoring message");
      res.status(400);
      res.send({
        error: "Invalid signature.",
      });
      return false;
    }
  }
}
